# -*- coding: utf-8 -*-
"""
BC SALTWATER INTRUSION RISK MAPPING
MARCH, 2021

NAME: aquifer_susceptibility.py

PURPOSE OF SCRIPT:

Create vector layers for aquifer susceptibility

AUTHOR:

Tim Sivak, MGISA, PGEO
tim@westernwater.ca

"""

import geopandas as gpd
import pandas as pd
from osgeo import gdal,ogr
import warnings
import time

# global variables

def calc_AqSusceptUNC(loc_in,area):
    gdal.UseExceptions()
    warnings.filterwarnings('ignore', 'GeoSeries.notna')
    reso = 150 #raster resolution in m
    #crs = {'init' :'epsg:3005'}
    #path = "D:\\Dropbox (WWA)\\WWAL\\Projects\\2019\\19-098-02VC Saltwater Intrusion Risk Mapping Year 2\\Data Analysis\\scripts\\"
    input_path = "D:\\Dropbox (WWA)\\WWAL\\Projects\\2019\\19-098-02VC Saltwater Intrusion Risk Mapping Year 2\\Data Analysis\\data_input\\"
    location = "D:\\Dropbox (WWA)\\WWAL\\Projects\\2019\\19-098-02VC Saltwater Intrusion Risk Mapping Year 2\\Data Analysis\\Pilot Areas\\Final Maps_mar\\" + area + "\\"
    
    #input files
    extent = location + "extent.shp"
    coast = input_path + "coast_edited.shp"  #must be line geometry
    gwells = location + "gwells_unc.shp" 
    grid = location + "grid.shp"
    #imaginary wells are "unconsolidated well points" that mark the overlap between aquifer boundary 
    #and coastline that are given unconsolidated susceptibility ratings of 5
    imaginary_wells = input_path + "imaginary_wells.shp" 

    
    #input_layer = location + "UNC_suscep.shp"
    
    #coastline buffers
    buffer_x1 = input_path + "buff_x1.shp"
    buffer_x2 = input_path + "buff_x2.shp"
    buffer_x3 = input_path + "buff_x3.shp"
    buffer_x4 = input_path + "buff_x4.shp"
    buffer_x5 = input_path + "buff_x5.shp"
    
    #output files
    #buffer_x1_cl = location + "buffer_x1_cl.shp"
    #buffer_x2_cl = location + "buffer_x2_cl.shp"
    #buffer_x3_cl = location + "buffer_x3_cl.shp"
    #buffer_x4_cl = location + "buffer_x4_cl.shp"
    #buffer_x5_cl = location + "buffer_x5_cl.shp"
    
    #output_tif = location + "UNC_suscept.tif"
    
    #--------------------- PREPARE DATASETS ----------------------------
    
    startTime = time.time()
   
    extent = gpd.GeoDataFrame.from_file(extent)
    coast = gpd.GeoDataFrame.from_file(coast)
    gwells = gpd.GeoDataFrame.from_file(gwells)
    grid = gpd.GeoDataFrame.from_file(grid)
    imaginary_wells = gpd.GeoDataFrame.from_file(imaginary_wells)
    
    buffer_x1 = gpd.GeoDataFrame.from_file(buffer_x1)
    buffer_x2 = gpd.GeoDataFrame.from_file(buffer_x2)
    buffer_x3 = gpd.GeoDataFrame.from_file(buffer_x3)
    buffer_x4 = gpd.GeoDataFrame.from_file(buffer_x4)
    buffer_x5 = gpd.GeoDataFrame.from_file(buffer_x5)
    
    #clip polygons to shape of extent
    
    buffer_x1 = gpd.clip(buffer_x1,extent)
    buffer_x2 = gpd.clip(buffer_x2,extent)
    buffer_x3 = gpd.clip(buffer_x3,extent)
    buffer_x4 = gpd.clip(buffer_x4,extent)
    buffer_x5 = gpd.clip(buffer_x5,extent)
    
    gwells = gpd.clip(gwells,extent)
    
    #set raster properties from extent
    xmin, ymin, xmax, ymax = extent.geometry.total_bounds
    
    width = int(round(xmax-xmin)/reso)
    height = int(round(ymax-ymin)/reso)
    
    #----------------------- AQUIFER SUSCEPTIBILITY ---------------------------------#
    
    #create buffer rasters in extent
    #print("Convert vector buffers into raster")
    driver = gdal.GetDriverByName('GTiff')
    
    buffer_x1.to_file(location + "buffer_x1_cl.shp")
    buffer_x2.to_file(location + "buffer_x2_cl.shp")
    buffer_x3.to_file(location + "buffer_x3_cl.shp")
    buffer_x4.to_file(location + "buffer_x4_cl.shp")
    buffer_x5.to_file(location + "buffer_x5_cl.shp")
    
    raster_list = [location + "buff_x1.tif",location + "buff_x2.tif", location + "buff_x3.tif",\
                   location + "buff_x4.tif",location + "buff_x5.tif"]
    source_list = [location + "buffer_x1_cl.shp",location + "buffer_x2_cl.shp",location + "buffer_x3_cl.shp",\
                   location + "buffer_x4_cl.shp",location + "buffer_x5_cl.shp"]
    
    i = 0
    
    for buffer in raster_list:
        raster_fn = raster_list[i]
        source_dataset = ogr.Open(source_list[i])
        source_layer = source_dataset.GetLayer()
        target_ds = driver.Create(raster_fn, width, height, 1, gdal.GDT_Int32)
        target_ds.SetGeoTransform((xmin, reso, 0, ymax, 0, -reso))
        buff = gdal.RasterizeLayer(target_ds, [1], source_layer,options=["ATTRIBUTE=hazard"])
        i += 1
    
    buffer_x1 = None
    buffer_x2 = None
    buffer_x3 = None
    buffer_x4 = None
    buffer_x5 = None
    target_ds = None
    driver = None
    
    #print("Calculate minimum distance of well points to coast")
    
    gwells['dist_to_coast'] = gwells.geometry.apply(lambda g: coast.distance(g).min())
    
    #TODO: move this to 'wells.py'???
    
    #set wells with no SWL to value of BOH
    gwells['static_wat'].fillna(gwells['finished_w'], inplace = True) 
    gwells['static_el'] = gwells['el'] - gwells['static_wat']
    
    gwells['hazard'] = 0
    
    print("Calculate aquifer susceptibility...")
    
    #define parameters from aquifer susceptibility matrix
    #based on report table
    # el_list is category for elevation of the static water level
    # dist_list is category for distance from coast
    
    el_list = [40,30,20,10]
    dist_list = [500,1000,1500,2000,3000]
    
    #define matrix row parameters from lists       
    el_row1 = el_list[0]
    el_row2 = el_list[1]
    el_row3 = el_list[2]
    el_row4 = el_list[3]
    
    # column 1
    
    #set distance based on parameter list
    dist_min = dist_list[0] 
    
    # set hazard column in GWELLS based on condition defined in "mask" variable
    
    mask = ((gwells['static_el'] > el_row1)&(gwells['dist_to_coast'] < dist_min))
    gwells['hazard'][mask] = 2
    
    mask = (((gwells['static_el'] > el_row2)&(gwells['static_el'] < el_row1))&(gwells['dist_to_coast']<dist_min))
    gwells['hazard'][mask] = 3
    
    mask = (((gwells['static_el'] > el_row3)&(gwells['static_el'] < el_row2))&(gwells['dist_to_coast']<dist_min))
    gwells['hazard'][mask] = 4
    
    mask = (((gwells['static_el'] > el_row4)&(gwells['static_el'] < el_row3))&(gwells['dist_to_coast']<dist_min))
    gwells['hazard'][mask] = 5
    
    mask = ((gwells['static_el'] < el_row4)&(gwells['dist_to_coast']<dist_min))
    gwells['hazard'][mask] = 5
    
    # column 2
    
    #set min and max distance based on parameter list
    dist_min = dist_list[0]
    dist_max = dist_list[1]
    
    mask = ((gwells['static_el'] > el_row1)&(gwells['dist_to_coast']>dist_min)&(gwells['dist_to_coast']<dist_max))
    gwells['hazard'][mask] = 1
    
    mask = (((gwells['static_el'] > el_row2)&(gwells['static_el']< el_row1))&((gwells['dist_to_coast']>dist_min)&(gwells['dist_to_coast']<dist_max)))
    gwells['hazard'][mask] = 2
    
    mask = (((gwells['static_el'] > el_row3)&(gwells['static_el']< el_row2))&((gwells['dist_to_coast']>dist_min)&(gwells['dist_to_coast']<dist_max)))
    gwells['hazard'][mask] = 3
    
    mask = (((gwells['static_el'] > el_row4)&(gwells['static_el']< el_row3))&((gwells['dist_to_coast']>dist_min)&(gwells['dist_to_coast']<dist_max)))
    gwells['hazard'][mask] = 4
    
    mask = ((gwells['static_el'] < el_row4)&((gwells['dist_to_coast']>dist_min)&(gwells['dist_to_coast']<dist_max)))
    gwells['hazard'][mask] = 5
    
    # column 3
    
    #set min and max distance based on parameter list
    dist_min = dist_list[1]
    dist_max = dist_list[2]
    
    mask = ((gwells['static_el'] > el_row1)&(gwells['dist_to_coast']>dist_min)&(gwells['dist_to_coast']<dist_max))
    gwells['hazard'][mask] = 1
    
    mask = (((gwells['static_el'] > el_row2)&(gwells['static_el']< el_row1))&((gwells['dist_to_coast']>dist_min)&(gwells['dist_to_coast']<dist_max)))
    gwells['hazard'][mask] = 2
    
    mask = (((gwells['static_el'] > el_row3)&(gwells['static_el']< el_row2))&((gwells['dist_to_coast']>dist_min)&(gwells['dist_to_coast']<dist_max)))
    gwells['hazard'][mask] = 3
    
    mask = (((gwells['static_el'] > el_row4)&(gwells['static_el']< el_row3))&((gwells['dist_to_coast']>dist_min)&(gwells['dist_to_coast']<dist_max)))
    gwells['hazard'][mask] = 4
    
    mask = ((gwells['static_el'] < el_row4)&((gwells['dist_to_coast']>dist_min)&(gwells['dist_to_coast']<dist_max)))
    gwells['hazard'][mask] = 5
    
    # column 4
    
    #set min and max distance based on parameter list
    dist_min = dist_list[2]
    dist_max = dist_list[3]
    
    mask = ((gwells['static_el'] > el_row1)&(gwells['dist_to_coast']>dist_min)&(gwells['dist_to_coast']<dist_max))
    gwells['hazard'][mask] = 1
    
    mask = (((gwells['static_el'] > el_row2)&(gwells['static_el']< el_row1))&((gwells['dist_to_coast']>dist_min)&(gwells['dist_to_coast']<dist_max)))
    gwells['hazard'][mask] = 1
    
    mask = (((gwells['static_el'] > el_row3)&(gwells['static_el']< el_row2))&((gwells['dist_to_coast']>dist_min)&(gwells['dist_to_coast']<dist_max)))
    gwells['hazard'][mask] = 2
    
    mask = (((gwells['static_el'] > el_row4)&(gwells['static_el']< el_row3))&((gwells['dist_to_coast']>dist_min)&(gwells['dist_to_coast']<dist_max)))
    gwells['hazard'][mask] = 3
    
    mask = ((gwells['static_el'] < el_row4)&((gwells['dist_to_coast']>dist_min)&(gwells['dist_to_coast']<dist_max)))
    gwells['hazard'][mask] = 4
    
    # column 5
    
    #set min and max distance based on parameter list
    dist_min = dist_list[3]
    dist_max = dist_list[4]
    
    mask = ((gwells['static_el'] > el_row1)&(gwells['dist_to_coast']>dist_min)&(gwells['dist_to_coast']<dist_max))
    gwells['hazard'][mask] = 1
    
    mask = (((gwells['static_el'] > el_row2)&(gwells['static_el']< el_row1))&((gwells['dist_to_coast']>dist_min)&(gwells['dist_to_coast']<dist_max)))
    gwells['hazard'][mask] = 1
    
    mask = (((gwells['static_el'] > el_row3)&(gwells['static_el']< el_row2))&((gwells['dist_to_coast']>dist_min)&(gwells['dist_to_coast']<dist_max)))
    gwells['hazard'][mask] = 1
    
    mask = (((gwells['static_el'] > el_row4)&(gwells['static_el']< el_row3))&((gwells['dist_to_coast']>dist_min)&(gwells['dist_to_coast']<dist_max)))
    gwells['hazard'][mask] = 2
    
    mask = ((gwells['static_el'] < el_row4)&((gwells['dist_to_coast']>dist_min)&(gwells['dist_to_coast']<dist_max)))
    gwells['hazard'][mask] = 3
       
    #set distance based on parameter list
    dist_min = dist_list[4] 
    
    #all wells > 'dist_min' from coast will have an aquifer susceptibility of zero
    
    mask = ((gwells['dist_to_coast']>dist_min))
    gwells['hazard'][mask] = 0
    
    grid = grid.dropna() #drop empty grids with no defined geometry
    
    #spatial join of wells to grid based on grid ID
    #set the hazard rating of the grid cell to be the highest hazard from wells assigned to the grid
    join = gpd.sjoin(grid, gwells, op='contains', how = 'right')
    max = join.groupby('Grid_ID').max()
    
    #create centroids of grid using maximum rating of well in cell
    centroids = grid.copy()
    centroids['geometry'] = centroids['geometry'].centroid
    centroids = centroids.merge(max, on='Grid_ID')
    centroids = centroids[['Grid_ID','hazard','geometry']]
    
    #clip imaginary wells to extent
    imaginary_wells = gpd.clip(imaginary_wells,extent)
    #merge imaginary wells with centroids
    centroids = gpd.GeoDataFrame(pd.concat([centroids,imaginary_wells]))
    
    centroids['UNC_sus'] = centroids['hazard']
    centroids = centroids[['Grid_ID','UNC_sus','geometry']]
    
    centroids.to_file(location + "UNC_suscep.shp")
      
    endTime = time.time()
    print("")
    print("Time elapsed: " + str((endTime - startTime)) + " seconds.")
