"""
BC SALTWATER INTRUSION RISK MAPPING
MARCH, 2021

Name: geoprocessing.py

Standard geoprocessing functions called from other modules

Tim Sivak, MGISA, P.Geo.
tim@westernwater.ca

"""
import geopandas as gpd
from shapely.geometry import Polygon, mapping
import numpy as np
from osgeo import gdal,osr
import fiona

def getRasterProperties(input):
    ds = gdal.Open(input)
    x_res = ds.RasterXSize
    y_res = ds.RasterYSize
    band = ds.GetRasterBand(1)
    prj = ds.GetProjection
    geotransform = ds.GetGeoTransform()
    wkt = ds.GetProjection()

    # setting spatial reference for output rasters
    srs = osr.SpatialReference()
    srs.ImportFromWkt(wkt)
    return x_res, y_res, band, prj, geotransform, wkt, srs

def clipPoly(input_lyr,mask_lyr):
    mask = mask_lyr.geometry.unary_union
    spatial_index = input_lyr.sindex
    
    # Create a box for the initial intersection
    bbox = mask.bounds
    
    # Get a list of id's for each line that overlaps the bounding box and subset the data to just those lines
    sidx = list(spatial_index.intersection(bbox))
    shp_sub = input_lyr.iloc[sidx]
    
    # Clip the data - with these data
    input_clip = shp_sub.copy()
    input_clip.buffer(0)
    input_clip['geometry'] = shp_sub.intersection(mask)
    
    # Return the clipped layer with no null geometry values
    return(input_clip[input_clip.geometry.notnull()])

def clipPoints(input_lyr, mask_lyr):
    mask = mask_lyr.geometry.unary_union
    return(input_lyr[input_lyr.geometry.intersects(mask)])

def createGrid(resolution,area):
    xmin,ymin,xmax,ymax =  area.total_bounds
    width = resolution
    height = resolution
    gridWidth = resolution
    gridHeight = resolution
    
    rows = int(np.ceil((ymax-ymin) /  height))
    cols = int(np.ceil((xmax-xmin) / width))
    XleftOrigin = xmin
    XrightOrigin = xmin + width
    YtopOrigin = ymax
    YbottomOrigin = ymax- height
    polygons = []
     
    for i in range(cols):
       Ytop = YtopOrigin
       Ybottom =YbottomOrigin
       for j in range(rows):
           polygons.append(Polygon([(XleftOrigin, Ytop), (XrightOrigin, Ytop), (XrightOrigin, Ybottom), (XleftOrigin, Ybottom)])) 
           Ytop = Ytop - gridHeight
           Ybottom = Ybottom - gridHeight
       XleftOrigin = XleftOrigin + gridWidth
       XrightOrigin = XrightOrigin + gridWidth
    
    #print (grid.crs)
    grid = gpd.GeoDataFrame({'geometry':polygons})
    return grid

def saveSHP(poly,out,ref,haz):
    # Define a polygon feature geometry with one attribute
    schema = {
        'geometry': 'Polygon',
        'properties': {'hazard': 'int'},
    }
    
    # Write a new Shapefile
    with fiona.open(out, 'w', 
                    crs=ref,
                    driver='ESRI Shapefile', 
                    schema=schema) as c:
        ## If there are multiple geometries, put the "for" loop here
        c.write({
            'geometry': mapping(poly),
            'properties': {'hazard': haz},
        })
        
def min_distance(point, lines):
    return lines.distance(point).min()

