# -*- coding: utf-8 -*-
"""
BC SALTWATER INTRUSION RISK MAPPING
MARCH, 2021

NAME: pumping_threat.py

PURPOSE OF SCRIPT:

1) create grid over extent of study area
2) import GWELLS and normalize attributes:
    a) convert yield into m3/day
    b) attribute wells by aquifer type based on ENV aquifer mapping, well completion, and quaternary geo coverage
    c) assign demand to each grid cell where wells are present, based on demand from GWELLs, and assumed / metered / licensed quantities
3) create Pumping Threat vector layers (grid centroids) depicting hazard for unconsolidated and bedrock aquifers
    
AUTHOR:

Tim Sivak, MGISA, PGEO
tim@westernwater.ca

"""

import geopandas as gpd
import pandas as pd
import warnings
import time

# custom functions
from wells import setAquifer
from geoprocessing import createGrid


# global variables

def calc_PumpThreat(gwells, loc_in, area):
       
    startTime = time.time()
    warnings.filterwarnings('ignore', 'GeoSeries.notna')
    reso = 150 #raster resolution in m
    crs = {'init' :'epsg:3005'}
    input_path = "D:\\Dropbox (WWA)\\WWAL\\Projects\\2019\\19-098-02VC Saltwater Intrusion Risk Mapping Year 2\\Data Analysis\\data_input\\"
    location = "D:\\Dropbox (WWA)\\WWAL\\Projects\\2019\\19-098-02VC Saltwater Intrusion Risk Mapping Year 2\\Data Analysis\\Pilot Areas\\Final Maps_mar\\" + area + "\\"
    
    #print("Pilot area: " + area)
    #print("Resolution = " + str(reso) + " m")
    
    #input files
    aquifers = input_path + "aquifers.csv"
    extent = location + "extent.shp"
    hatcheries = input_path + "hatcheries.shp"
    hatcheries_noWTN = input_path + "hatcheries_noWTN.shp"
    water_rights = input_path + "water_rights.shp"
    dwss = input_path + "dwss.shp"
    dwss_noWTN = input_path + 'dwss_noWTN.shp'
    quat_geo = input_path + "quat_geo.shp"
      
    #output files
    gwells_unc_out = location + "gwells_unc.shp"
    gwells_bed_out = location + "gwells_bed.shp"
    grid_out = location + "grid.shp"
    pumpingthreat_unc = location + "pumping_threat_unc.shp"
    pumpingthreat_bed = location + "pumping_threat_bed.shp"
    
    #--------------------- PREPARE DATASETS ----------------------------
    
    #load dataframes.  extent will be NTS mapsheet in the future
    #print("Load input datasets...")
    extent = gpd.GeoDataFrame.from_file(extent)
    hatcheries = gpd.GeoDataFrame.from_file(hatcheries)
    hatcheries_noWTN = gpd.GeoDataFrame.from_file(hatcheries_noWTN)
    water_rights = gpd.GeoDataFrame.from_file(water_rights)
    dwss = gpd.GeoDataFrame.from_file(dwss)
    dwss_noWTN = gpd.GeoDataFrame.from_file(dwss_noWTN)
    quat_geo = gpd.GeoDataFrame.from_file(quat_geo)

    
    #create grid that matches bounding box of shapefile
    #print("Create grid...")
    grid = createGrid(reso,extent)
    grid.crs = crs
    
    #clip polygons to shape of extent
    #print ("Clip polygons to extent...")
    grid_clipped = gpd.clip(grid,extent)
    quat_geo = gpd.clip(quat_geo,extent)
  
    #print ("Clip point layers to extent...")
    gwells = gpd.clip(gwells,extent)
    hatcheries_clipped = gpd.clip(hatcheries,extent)
    hatcheries_noWTN_clipped = gpd.clip(hatcheries_noWTN, extent)
    water_rights_clipped = gpd.clip(water_rights, extent)
    dwss_clipped = gpd.clip(dwss, extent)
    dwss_noWTN_clipped = gpd.clip(dwss_noWTN, extent)
    
    #print ("Import layers as geodataframes...")
    gwells = gpd.GeoDataFrame(gwells, crs=crs)
    gwells_raw = gwells #back up full well record
    hatcheries_clipped = gpd.GeoDataFrame(hatcheries_clipped, crs=crs)
    hatcheries_noWTN_clipped = gpd.GeoDataFrame(hatcheries_noWTN_clipped, crs=crs)
    water_rights_clipped = gpd.GeoDataFrame(water_rights_clipped, crs=crs)
    grid_clipped = gpd.GeoDataFrame(grid_clipped, crs=crs)
    dwss_clipped = gpd.GeoDataFrame(dwss_clipped, crs=crs)
    
    print ("Assign unique ID to each grid cell...")
    grid_clipped.insert(1,'Grid_ID', range(0, len(grid_clipped)))
    grid_clipped.to_file(grid_out)
    
    print ("Attribute well as completed in unconsolidated sediments (UNC) or fractured bedrock (BED)")
    
    gwells = setAquifer(gwells,aquifers,quat_geo,input_path,\
                        location,dwss_clipped,hatcheries_clipped,water_rights_clipped,\
                        dwss_noWTN_clipped,hatcheries_noWTN_clipped)
    gwells.to_file(location + "\\" + "gwells_parsed.shp")
    
    GWELLS_unc = gwells.loc[gwells['type'] == 'UNC']
    GWELLS_bed = gwells.loc[gwells['type'] == 'BED']
    
    '''
    print("")
    print("Sample of GWELLS (UNC):")
    print("")
    print(GWELLS_unc.head(5))
    
    print("")
    print("Sample of GWELLS (BED):")
    print("")
    print(GWELLS_bed.head(5))
    '''
    
    #----------------------- PUMPING THREAT ---------------------------------#
    print ("")
    print ("Create Pumping Threat Centroids for UNC and BED")
    
    gwells_raw['WTN'] = gwells_raw['well_tag_n']
    gwells_raw = gwells_raw[['WTN','el','finished_w','static_wat']]
    gwells_raw = gwells_raw.set_index('WTN')
    
    #Unconsolidated centroids
    join_unc = gpd.sjoin(grid_clipped, GWELLS_unc, op='contains', how = 'right')
    sum = join_unc.groupby('Grid_ID').sum()
        
    centroids_unc = grid_clipped.copy()
    centroids_unc['geometry'] = centroids_unc['geometry'].centroid
        
    centroids_unc = centroids_unc.merge(sum, on='Grid_ID')
    centroids_unc['pumpinghaz'] = None
    centroids_unc = centroids_unc[['Grid_ID','demand','geometry']]
    
    #bedrock centroids
    join_bed = gpd.sjoin(grid_clipped, GWELLS_bed, op='contains', how = 'right')
    sum = join_bed.groupby('Grid_ID').sum()
        
    centroids_bed = grid_clipped.copy()
    centroids_bed['geometry'] = centroids_bed['geometry'].centroid
        
    centroids_bed = centroids_bed.merge(sum, on='Grid_ID')
    centroids_bed['pumpinghaz'] = None
    centroids_bed = centroids_bed[['Grid_ID','demand','geometry']]
    
    centroids_unc.to_file(pumpingthreat_unc)
    centroids_bed.to_file(pumpingthreat_bed)
    
    #parse and save unconsolidated wells for aquifer susceptibility function  
    GWELLS_aq_unc = GWELLS_unc.set_index('WTN')
    GWELLS_aq_unc = pd.merge(GWELLS_aq_unc,gwells_raw, how='left', on='WTN')  
    GWELLS_aq_unc = GWELLS_aq_unc.reset_index()
    
    GWELLS_aq_unc = gpd.GeoDataFrame(GWELLS_aq_unc)
    GWELLS_aq_unc = GWELLS_aq_unc[['WTN','demand','type','finished_w','static_wat','el','geometry']]
    GWELLS_aq_unc['finished_w'] = GWELLS_aq_unc['finished_w'] * 0.3048 #convert ft to meters
    GWELLS_aq_unc['static_wat'] = GWELLS_aq_unc['static_wat'] * 0.3048 #convert ft to meters
    
    GWELLS_bed.to_file(gwells_bed_out)
    GWELLS_aq_unc.to_file(gwells_unc_out)
    
    endTime = time.time()
    print("")
    print("Time elapsed: " + str((endTime - startTime)) + " seconds.")
    
