# -*- coding: utf-8 -*-
"""
BC SALTWATER INTRUSION RISK MAPPING
MARCH, 2021

NAME: wells.py

PURPOSE OF SCRIPT:

import GWELLS and normalize attributes:
    a) convert yield into m3/day
    b) attribute wells by aquifer type based on ENV aquifer mapping, well completion, and quaternary geo coverage
    c) assign demand to each grid cell where wells are present, based on demand from GWELLs, and assumed / metered / licensed quantities
    
AUTHOR:

Tim Sivak, MGISA, P.Geo.
tim@westernwater.ca

"""

import pandas as pd
import geopandas as gpd
import numpy as np

def setAquifer(gwells,aquifers,quat_geo,input_path,location,dwss\
               ,hatcheries,water_rights,dwss_noWTN,hatcheries_noWTN):   
    
    aquifers = pd.read_csv(aquifers)    
    aquifers = aquifers[['aquifer_id','material']]
    #aquifers['type'] = np.nan
    #aquifers['type'] = "UNK"
    
    aquifers['type'] = np.where(aquifers['material']=="B - Bedrock","BED",\
                np.where(aquifers['material']=="SG - Sand and Gravel","UNC",\
                np.where(aquifers['material']=="S - Sand","UNC",\
                np.where(aquifers['material']=="G - Gravel","UNC","UNK"))))
           
    gwells['type'] = gwells['aquifer_id'].map(aquifers.set_index('aquifer_id')['type'])
    
    gwells['type'].fillna("None", inplace = True)
    print("")  
    
    print("set wells to BEDROCK when finished well depth is greater than bedrock depth...")
    mask = (gwells['finished_w']) <= ((gwells['bedrock_de']))
    gwells['type'][mask] = "UNC"
    
    print("")  
    
    print ("set deeper wells to BED, based on well depth statistics")
    print ("~400 ft is the ~90% percentile for wells classified as bedrock based on aquifer ID")
    
    isUNC_or_BED = 400 #in feet 
    mask = (gwells['finished_w'] >= isUNC_or_BED)&((gwells['type'] == "None") | (gwells['type'] == np.nan) | (gwells['type'] == "UNK"))
    gwells['type'][mask] = "BED"
       
    print("")    
    print("Unclassified wells inside quat_geo poly are set to UNC, and those outside are set to BED")
    print("This is required because some wells don't have a final depth")
        
    quat_geo = quat_geo.geometry.unary_union 
           
    within_quatgeo = gwells[gwells.geometry.within(quat_geo)]
    mask = (within_quatgeo['type'] == "None") | (within_quatgeo['type'] == "UNK") | (gwells['type'] == np.nan)
    within_quatgeo['type'][mask] = "UNC"
        
    outside_quatgeo = gwells[~gwells.geometry.within(quat_geo)]
    mask = (outside_quatgeo['type'] == "None") | (outside_quatgeo['type'] == "UNK") | (gwells['type'] == np.nan)
    outside_quatgeo['type'][mask] = "BED"
        
    frames = [within_quatgeo, outside_quatgeo]
    
    isUNC_or_BED = 20 #in feet      
    print("Set shallow wells to UNC and deep wells to BED based on finished well depth: " + str(isUNC_or_BED) + " ft")    
    mask = (gwells['finished_w'] <= isUNC_or_BED)&((gwells['type'] == "None") | (gwells['type'] == np.nan) | (gwells['type'] == "UNK"))
    gwells['type'][mask] = "UNC"
    
    #concatenate gwells data outside and inside quaternary geology shapes
    gwells = pd.concat(frames)
    
    gwells['demand'] = None 
    gwells['WTN'] = gwells['well_tag_n'].astype(int)
    
       
    print("Convert gwells yield to m3/day; give wells with empty yield field a demand of 2 m3/day")
    print("Wells with no yield unit are assumed to be USgpm")
            
    gwells['demand'] = np.where(gwells['well_yie_1']=="GPH", gwells['well_yield']*0.0908499,\
              np.where(gwells['well_yie_1']=="GPM", gwells['well_yield']*5.45099,\
              np.where(gwells['well_yie_1']=="USGPM", gwells['well_yield']*5.45099,\
              np.where(gwells['well_yie_1']=="IGM", gwells['well_yield']*6.54637,\
              np.where(gwells['well_yie_1']=="UNK", gwells['well_yield']*5.45099,\
              np.where(gwells['well_yie_1']=="LPS", gwells['well_yield']*86.4,\
              np.where(gwells['well_yie_1']=="DRY", 0,\
              np.where(gwells['well_yie_1']=="",gwells['well_yield']*5.45099,\
              np.where(gwells['well_yie_1']== np.nan,0,gwells['well_yield'])))))))))
        
        
    print("If the well is domestic, unknown or blank use, then set demand to 2 m3/day, otherwise leave as yield.")
        
    gwells.loc[gwells.intended_w == 'DOM', 'demand'] = 2
    gwells.loc[gwells.intended_w == 'UNK', 'demand'] = 2
    gwells.loc[gwells.intended_w == '', 'demand'] = 2
    gwells['demand'] = gwells['demand'].replace(np.nan,2)
    gwells['demand'].fillna('2', inplace=True)
    
    #DEPRECIATED
    #print("Pull out intended use for loss component")

    #gwells_use = gwells[['WTN','demand','type','intended_w','geometry']]
    #gwells_use.to_file(location + "gwells_use.shp") #export domestic wells (for loss module)   
    
    gwells = gwells[['WTN','demand','type','intended_w','geometry']]
    
    #DEPRECIATED
    #print("Pull out domestic wells for loss component")
    #gwells_domestic = (gwells['intended_w'] == 'DOM' | gwells['intended_w'] == 'UNK' | gwells['intended_w'] == '')
    
    #create df for wells by WTN and aquifer type
    gwells_type = gwells.set_index('WTN')
    gwells_type = gwells_type[['type']]
      
    
    dwss_list = dwss['WTN'].tolist()
    dwss = dwss.set_index('WTN')
    dwss['intended_w'] = "DWSS"
    dwss = pd.merge(dwss,gwells_type, how='left', on='WTN')
    dwss = dwss.reset_index()
    dwss = dwss[['WTN','demand','type','intended_w','geometry']]
    
    dwss_noWTN['WTN']=dwss_noWTN['wtn']
    dwss_noWTN['intended_w'] = "DWSS"
    dwss_noWTN = dwss_noWTN[['WTN','demand','type','intended_w','geometry']]
    
    hatcheries_noWTN['intended_w'] = "HATCHERY"
    hatcheries_noWTN = hatcheries_noWTN[['WTN','demand','type','intended_w','geometry']]
        
    hatcheries_list = hatcheries['WTN'].tolist()
    hatcheries['intended_w'] = "HATCHERY"
    hatcheries = hatcheries.set_index('WTN')
    hatcheries = pd.merge(hatcheries,gwells_type, how='left', on='WTN')
    hatcheries = hatcheries.reset_index()
    hatcheries['type'] = hatcheries['type_y']
    hatcheries = hatcheries[['WTN','demand','type','intended_w','geometry']]
    
    #remove water rights points that have no well tag nunbers, therefore only considering PWD
    #replace well points with water right by corresponding WTN
    water_rights.dropna(subset=['WLL_TG_NMR'], inplace=True)
    water_rights['intended_w'] = "LICENCE"
    water_rights['WTN'] = water_rights['WLL_TG_NMR'].astype(int)
    water_rights_list = water_rights['WTN'].tolist()
    water_rights['demand'] = None
    water_rights['type'] = None
    water_rights['demand'] = np.where(water_rights['QNTTY_NTS']=="m3/year",water_rights['QUANTITY']/365,\
                np.where(water_rights['QNTTY_NTS']=="m3/sec",water_rights['QUANTITY']*86400, water_rights['QUANTITY']))
    
    water_rights = water_rights [['WTN','demand','type','intended_w','geometry']]
    water_rights = water_rights.set_index('WTN')
    water_rights = pd.merge(water_rights,gwells_type, how='left', on='WTN')
    water_rights = water_rights.reset_index()
    water_rights['type'] = water_rights['type_y']
    water_rights = water_rights[['WTN','demand','type','intended_w','geometry']]
    
    #remove hatcheries and dwss from gwells
    gwells = gwells[~gwells['WTN'].isin(water_rights_list)]
    gwells = gwells[~gwells['WTN'].isin(hatcheries_list)]
    gwells = gwells[~gwells['WTN'].isin(dwss_list)]
    
    #fill all remaining empty aquifer type as BED, except for hatchery because they demand greater water than typically available in BED
    gwells['type'].fillna("BED", inplace=True)
    dwss['type'].fillna("BED", inplace=True)
    hatcheries['type'].fillna("UNC", inplace=True)
    water_rights['type'].fillna("BED", inplace=True)
    
    frames = [dwss, dwss_noWTN]
    dwss = pd.concat(frames)
    
    frames = [hatcheries, hatcheries_noWTN]
    hatcheries = pd.concat(frames)
    
    #append hatcheries, dwss, gwells and water rights dataframes
    frames = [dwss, hatcheries, water_rights, gwells]
    gwells = pd.concat(frames)
            
    mask = (gwells['demand'] == 0) #set wells with 0 demand to 2 m3/d
    gwells['demand'][mask] = 2
    
    #gwells.to_file(location + "\\" + "pumpingthreat.shp")
    
    return gwells
